@@ -121,7 +121,7 @@
 
 #define SASL_VERSION_MAJOR 2
 #define SASL_VERSION_MINOR 1
-#define SASL_VERSION_STEP 6
+#define SASL_VERSION_STEP 7
 
 #include "prop.h"
 #define LIBSASL_API
@@ -257,6 +257,7 @@ typedef unsigned sasl_ssf_t;
 /* usage flags provided to sasl_server_new and sasl_client_new:
  */
 #define SASL_SUCCESS_DATA    0x0004 /* server supports data on success */
+#define SASL_NEED_PROXY      0x0008 /* require a mech that allows proxying */
 
 /***************************
  * Security Property Types *
@@ -699,6 +700,7 @@ LIBSASL_API int sasl_getprop(sasl_conn_t *conn, int propnum,
 #define SASL_AUTHSOURCE   14	/* name of auth source last used, useful
 				 * for failed authentication tracking */
 #define SASL_MECHNAME     15    /* active mechanism name, if any */
+#define SASL_AUTHUSER     16    /* authentication/admin user */
 
 /* This returns a string which is either empty or has an error message
  * from sasl_seterror (e.g., from a plug-in or callback).  It differs
