@@ -25,7 +25,6 @@ OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 ******************************************************************/
 
 #include <config.h>
-
 #include <stdio.h>
 #include <sasl.h>
 #include <saslutil.h>
@@ -47,17 +46,17 @@ static int setcomplete(JNIEnv *env, jobject obj);
 
 static void throwexception(JNIEnv *env, int error)
 {
-    jclass newExcCls;
+  jclass newExcCls;
 
-    VL (("Throwing exception!\n"));
+  VL (("Throwing exception!\n"));
 
-    newExcCls = (*env)->FindClass(env, "CyrusSasl/SaslException");
+  newExcCls = (*env)->FindClass(env, "CyrusSasl/SaslException");
 
-    if (newExcCls == 0) { 
-	return;
-    }
+  if (newExcCls == 0) { 
+    return;
+  }
 
-    (*env)->ThrowNew(env, newExcCls, sasl_errstring(error,NULL,NULL));
+  (*env)->ThrowNew(env, newExcCls, sasl_errstring(error,NULL,NULL));
 }
 
 /* server init */
@@ -67,18 +66,18 @@ JNIEXPORT jint JNICALL Java_CyrusSasl_ServerFactory_jni_1sasl_1server_1init
    jobject obj __attribute__((unused)),
    jstring jstr)
 {
-    /* Obtain a C-copy of the Java string */
-    const char *str = (*env)->GetStringUTFChars(env, jstr, 0);
-    int result;
+  /* Obtain a C-copy of the Java string */
+  const char *str = (*env)->GetStringUTFChars(env, jstr, 0);
+  int result;
 
-    result=sasl_server_init(NULL,str);
-    if (result!=SASL_OK)
-	throwexception(env,result);
+  result=sasl_server_init(NULL,str);
+  if (result!=SASL_OK)
+    throwexception(env,result);
 
-    /* Now we are done with str */
-    (*env)->ReleaseStringUTFChars(env, jstr, str);
+  /* Now we are done with str */
+  (*env)->ReleaseStringUTFChars(env, jstr, str);
 
-    return result;
+  return result;
 }
 
 static int
@@ -91,7 +90,6 @@ log(void *context __attribute__((unused)),
   jclass cls;
   jmethodID mid;
 
-
   if (! message)
     return SASL_BADPARAM;
 
@@ -99,17 +97,30 @@ log(void *context __attribute__((unused)),
   case SASL_LOG_ERR:
     label = "Error";
     break;
-  case SASL_LOG_WARNING:
+  case SASL_LOG_WARN:
     label = "Warning";
     break;
-  case SASL_LOG_INFO:
-    label = "Info";
+  case SASL_LOG_NOTE:
+    label = "Note";
+    break;
+  case SASL_LOG_FAIL:
+    label = "Fail";
+    break;
+  case SASL_LOG_PASS:
+    label = "Pass";
+    break;
+  case SASL_LOG_TRACE:
+    label = "Trace";
+    break;
+  case SASL_LOG_DEBUG:
+    label = "Debug";
     break;
   default:
     return SASL_BADPARAM;
   }
 
-  VL(("Trying to call password callback\n"));
+  VL(("I have message %s\n",message));
+  VL(("Trying to call log callback\n"));
   cls = (*globalenv)->GetObjectClass(globalenv, globalobj);
   mid = (*globalenv)->GetMethodID(globalenv, cls, "callback_log",
 				  "(Ljava/lang/String;Ljava/lang/String;)V");
@@ -133,6 +144,8 @@ log(void *context __attribute__((unused)),
   /* Now we are done with jmessage */
   (*globalenv)->ReleaseStringUTFChars(globalenv, jmessage, message);
 
+  VL(("done with log callback"));
+
   return SASL_OK;
 }
 
@@ -161,20 +174,20 @@ JNIEXPORT jint JNICALL Java_CyrusSasl_ClientFactory_jni_1sasl_1client_1init
    jobject obj __attribute__((unused)),
    jstring jstr)
 {
-    /* Obtain a C-copy of the Java string */
-    const char *str = (*env)->GetStringUTFChars(env, jstr, 0);
-    int result;
+  /* Obtain a C-copy of the Java string */
+  const char *str = (*env)->GetStringUTFChars(env, jstr, 0);
+  int result;
 
-    VL(("client initing\n"));
+  VL(("client initing\n"));
 
-    result=sasl_client_init(callbacks);
-    if (result!=SASL_OK)
-	throwexception(env,result);
+  result=sasl_client_init(callbacks);
+  if (result!=SASL_OK)
+    throwexception(env,result);
 
-    /* Now we are done with str */
-    (*env)->ReleaseStringUTFChars(env, jstr, str);
+  /* Now we are done with str */
+  (*env)->ReleaseStringUTFChars(env, jstr, str);
 
-    return result;
+  return result;
 }
 
 /* server new */
@@ -186,30 +199,30 @@ JNIEXPORT jint JNICALL Java_CyrusSasl_ServerFactory_jni_1sasl_1server_1new
    jstring jlocal, 
    jint jsecflags)
 {
-    sasl_conn_t *conn;
+  sasl_conn_t *conn;
 
-    const char *service = (*env)->GetStringUTFChars(env, jservice, 0);
-    const char *local_domain = (*env)->GetStringUTFChars(env, jlocal, 0);
-    const char *user_domain = NULL;  
-    int result;
+  const char *service = (*env)->GetStringUTFChars(env, jservice, 0);
+  const char *local_domain = (*env)->GetStringUTFChars(env, jlocal, 0);
+  const char *user_domain = NULL;  
+  int result;
 
-    if (local_domain) {
-	VL(("local domain = %s\n",local_domain));
-    }
-    if (user_domain) {
-	VL(("user domain = %s\n",user_domain));
-    }
+  if (local_domain) {
+      VL(("local domain = %s\n",local_domain));
+  }
+  if (user_domain) {
+      VL(("user domain = %s\n",user_domain));
+  }
 
-    result=sasl_server_new(service, local_domain, user_domain, 
-			   NULL, jsecflags, &conn);
-    if (result!=SASL_OK)
-	throwexception(env,result);
+  result=sasl_server_new(service, local_domain, user_domain, 
+			 NULL, NULL, NULL, jsecflags, &conn);
+  if (result!=SASL_OK)
+    throwexception(env,result);
 
-    /* Now we are done with str */
-    (*env)->ReleaseStringUTFChars(env, jservice, service);  
-    (*env)->ReleaseStringUTFChars(env, jlocal, local_domain);  
+  /* Now we are done with str */
+  (*env)->ReleaseStringUTFChars(env, jservice, service);  
+  (*env)->ReleaseStringUTFChars(env, jlocal, local_domain);  
 
-    return (jint) conn;
+  return (jint) conn;
 }
 
 
@@ -218,22 +231,23 @@ JNIEXPORT jint JNICALL JNICALL Java_CyrusSasl_ClientFactory_jni_1sasl_1client_1n
    jobject obj __attribute__((unused)),
    jstring jservice, jstring jserver, jint jsecflags)
 {
-    sasl_conn_t *conn;
+  sasl_conn_t *conn;
 
-    const char *service = (*env)->GetStringUTFChars(env, jservice, 0);
-    const char *serverFQDN = (*env)->GetStringUTFChars(env, jserver, 0);
-    int result;
+  const char *service = (*env)->GetStringUTFChars(env, jservice, 0);
+  const char *serverFQDN = (*env)->GetStringUTFChars(env, jserver, 0);
+  int result;
 
-    result=sasl_client_new(service, serverFQDN, NULL, jsecflags, &conn);
+  result=sasl_client_new(service, serverFQDN, NULL, NULL, NULL,
+			 jsecflags, &conn);
 
-    if (result!=SASL_OK)
-	throwexception(env,result);
+  if (result!=SASL_OK)
+    throwexception(env,result);
 
-    /* Now we are done with str */
-    (*env)->ReleaseStringUTFChars(env, jservice, service);  
-    (*env)->ReleaseStringUTFChars(env, jserver, serverFQDN);  
+  /* Now we are done with str */
+  (*env)->ReleaseStringUTFChars(env, jservice, service);  
+  (*env)->ReleaseStringUTFChars(env, jserver, serverFQDN);  
 
-    return (jint) conn;
+  return (jint) conn;
 }
 
 /* server start */
@@ -243,37 +257,47 @@ JNIEXPORT jbyteArray JNICALL Java_CyrusSasl_GenericServer_jni_1sasl_1server_1sta
    jobject obj __attribute__((unused)),
    jint ptr, jstring jstr, jbyteArray jarr, jint jlen)
 {
-    sasl_conn_t *conn;
-    const char *mech = (*env)->GetStringUTFChars(env, jstr, 0);
-    char *out;
-    unsigned int outlen;
-    int result;
-    jbyteArray arr;
-    const char *errstr;
-    char *in=NULL;
+  sasl_conn_t *conn;
+  const char *mech = (*env)->GetStringUTFChars(env, jstr, 0);
+  const char *out;
+  unsigned int outlen;
+   int result;
+  jbyteArray arr;
+  char *tmp;
+  char *in=NULL;
 
-    VL(("in server start\n"));
+  VL(("in server start\n"));
 
-    if (jarr!=NULL)
-	in = (*env)->GetByteArrayElements(env, jarr, 0);
+  if (jarr!=NULL)
+    in = (*env)->GetByteArrayElements(env, jarr, 0);
 
-    conn=(sasl_conn_t *) ptr;
+  conn=(sasl_conn_t *) ptr;
 
-    result=sasl_server_start(conn, mech,
-			     (const char *) in, jlen,
-			     &out, &outlen,
-			     &errstr);
+  result=sasl_server_start(conn, mech,
+			   (const char *) in, jlen,
+			   &out, &outlen);
 
-    if ((result!=SASL_OK) && (result!=SASL_CONTINUE)) {
-	throwexception(env,result);
-	return NULL;
-    }
+  if ((result!=SASL_OK) && (result!=SASL_CONTINUE))
+  {
+
+    throwexception(env,result);
+    return NULL;
+  }
+
+  /* Because SASLv2 does not allow for persistance, we'll copy
+   * it here */
+  tmp = malloc(outlen);
+  if(!tmp) {
+      throwexception(env, SASL_NOMEM);
+      return NULL;
+  }
+
+  memcpy(tmp, out, outlen);
   
+  arr=(*env)->NewByteArray(env,outlen);
+  (*env)->SetByteArrayRegion(env,arr, 0, outlen, (char *)tmp);
 
-    arr = (*env)->NewByteArray(env, outlen);
-    (*env)->SetByteArrayRegion(env, arr, 0, outlen, (char *) out);
-
-    return arr;
+  return arr;
 }
 
 
@@ -286,7 +310,8 @@ static int getvalue(JNIEnv *env, jobject obj, char *funcname, char **result, int
     
     /* set up for java callback */
     cls = (*env)->GetObjectClass(env, obj);
-    mid = (*env)->GetMethodID(env, cls, funcname, "(I)Ljava/lang/String;");
+    mid = (*env)->GetMethodID(env, cls, funcname,
+				  "(I)Ljava/lang/String;");
     if (mid == 0) {
 	VL(("Can't find %s callback!!!\n",funcname));
 	return SASL_FAIL;
@@ -387,66 +412,75 @@ static int fillin_interactions(JNIEnv *env, jobject obj,
 JNIEXPORT jbyteArray JNICALL Java_CyrusSasl_GenericClient_jni_1sasl_1client_1start
   (JNIEnv *env, jobject obj, jint ptr, jstring jstr)
 {    
-    sasl_conn_t *conn=(sasl_conn_t *) ptr;
-    const char *mechlist = (*env)->GetStringUTFChars(env, jstr, 0);
-    char *out;
-    unsigned int outlen=0;
-    const char *mechusing;
-    int result;
-    sasl_interact_t *client_interact=NULL;
-    jbyteArray arr;
-    jstring jmechusing;
-    jclass cls;
-    jmethodID mid;
+  sasl_conn_t *conn=(sasl_conn_t *) ptr;
+  const char *mechlist = (*env)->GetStringUTFChars(env, jstr, 0);
+  const char *out;
+  unsigned int outlen=0;
+  const char *mechusing;
+  int result;
+  sasl_interact_t *client_interact=NULL;
+  char *tmp;
+  jbyteArray arr;
+  jstring jmechusing;
+  jclass cls;
+  jmethodID mid;
 
-    VL(("sasl_start"));
+  VL(("sasl_start"));
  
-    do {
+  do {
 
-	result=sasl_client_start(conn, mechlist,
-				 NULL, &client_interact,
-				 &out, &outlen,
-				 &mechusing);
+      result=sasl_client_start(conn, mechlist,
+			       &client_interact,
+			       &out, &outlen,
+			       &mechusing);
 
-	if (result==SASL_INTERACT)
-	    result=fillin_interactions(env,obj,client_interact);
+      if (result==SASL_INTERACT)
+	  result=fillin_interactions(env,obj,client_interact);
 
-    } while (result==SASL_INTERACT);
+  } while (result==SASL_INTERACT);
 
-    /* ok release mechlist */
-    (*env)->ReleaseStringUTFChars(env, jstr, mechlist);  
+  /* ok release mechlist */
+  (*env)->ReleaseStringUTFChars(env, jstr, mechlist);  
 
-    if ((result!=SASL_OK) && (result!=SASL_CONTINUE))
-	{
-	    throwexception(env,result);
-	    return NULL;
-	}
+  if ((result!=SASL_OK) && (result!=SASL_CONTINUE))
+  {
+    throwexception(env,result);
+    return NULL;
+  }
 
+  /* tell the java layer what mechanism we're using */
 
-    /* tell the java layer what mechanism we're using */
+  /* set up for java callback */
+  cls = (*env)->GetObjectClass(env, obj);
+  mid = (*env)->GetMethodID(env, cls, "callback_setmechanism",
+			    "(Ljava/lang/String;)V");
+  if (mid == 0) {
+      throwexception(env,SASL_FAIL);
+    return NULL;
+  }
 
-    /* set up for java callback */
-    cls = (*env)->GetObjectClass(env, obj);
-    mid = (*env)->GetMethodID(env, cls, "callback_setmechanism",
-			      "(Ljava/lang/String;)V");
-    if (mid == 0) {
-	throwexception(env,SASL_FAIL);
-	return NULL;
-    }
+  VL(("mechusing=%s\n",mechusing));
 
-    VL(("mechusing=%s\n",mechusing));
+  /* make into mech */
+  jmechusing= (*env)->NewStringUTF(env,mechusing);
 
-    /* make into mech */
-    jmechusing = (*env)->NewStringUTF(env, mechusing);
+  /* do the callback */
+  (*env)->CallVoidMethod(env, obj, mid,jmechusing);
 
-    /* do the callback */
-    (*env)->CallVoidMethod(env, obj, mid,jmechusing);
+  /* Because SASLv2 does not allow for persistance, we'll copy
+   * it here */
+  tmp = malloc(outlen);
+  if(!tmp) {
+      throwexception(env, SASL_NOMEM);
+      return NULL;
+  }
 
-    /* make the byte array to return */
-    arr=(*env)->NewByteArray(env, outlen);
-    (*env)->SetByteArrayRegion(env, arr, 0, outlen, (char *) out);
+  memcpy(tmp, out, outlen);
   
-    return arr;
+  arr=(*env)->NewByteArray(env,outlen);
+  (*env)->SetByteArrayRegion(env,arr, 0, outlen, (char *)tmp);
+  
+  return arr;
 }
 
 /* server step */
@@ -459,18 +493,17 @@ JNIEXPORT jbyteArray JNICALL Java_CyrusSasl_GenericServer_jni_1sasl_1server_1ste
 {
   sasl_conn_t *conn=(sasl_conn_t *) ptr;
   int result;
-  char *out;
+  const char *out;
   unsigned int outlen;
   jbyteArray arr;
-  const char *errstr;  
   char *in = NULL;
-
-  if (jlen > 0) {
+  char *tmp;
+  
+  if (jlen > 0)
       in = (*env)->GetByteArrayElements(env, jarr, 0);
-  }
 
-  result = sasl_server_step(conn, in, jlen,
-			    &out, &outlen, &errstr);
+  result=sasl_server_step(conn, (const char *) in, jlen,
+			  &out, &outlen);
 
   if ((result!=SASL_OK) && (result!=SASL_CONTINUE))
   {
@@ -484,14 +517,21 @@ JNIEXPORT jbyteArray JNICALL Java_CyrusSasl_GenericServer_jni_1sasl_1server_1ste
       setcomplete(env,obj);
   }
 
-  if (jlen > 0) {
-      (*env)->ReleaseByteArrayElements(env, jarr, in, 0);
+  if (jlen > 0)
+      (*env)->ReleaseByteArrayElements(env, jarr,in ,0);
+
+  /* Because SASLv2 does not allow for persistance, we'll copy
+   * it here */
+  tmp = malloc(outlen);
+  if(!tmp) {
+      throwexception(env, SASL_NOMEM);
+      return NULL;
   }
 
-  /* make byte array to return with stuff to send to server */
-  arr = (*env)->NewByteArray(env, outlen);
-
-  (*env)->SetByteArrayRegion(env, arr, 0, outlen, (char *) out);
+  memcpy(tmp, out, outlen);
+  
+  arr=(*env)->NewByteArray(env,outlen);
+  (*env)->SetByteArrayRegion(env,arr, 0, outlen, (char *)tmp);
 
   return arr;
 }
@@ -528,28 +568,32 @@ JNIEXPORT jbyteArray JNICALL Java_CyrusSasl_GenericClient_jni_1sasl_1client_1ste
     (JNIEnv *env, jobject obj, jint ptr, jbyteArray jarr, jint jlen)
 {    
   sasl_conn_t *conn=(sasl_conn_t *) ptr;
+  /*  const char *in = (*env)->GetStringUTFChars(env, jstr, 0);*/
   int result;
   sasl_interact_t *client_interact=NULL;
-  char *out;
+  const char *out;
   unsigned int outlen;
   jbyteArray arr;
   char *in;
-
+  char *tmp;
+  
   VL(("in client step\n"));
 
   if (jarr) {
       in = (*env)->GetByteArrayElements(env, jarr, 0);
+      in[jlen]=0;
   } else {
       assert(jlen == 0);
-      in = NULL;
+       in = NULL;
   }
 
   globalenv=env;
   globalobj=obj;
 
   do {
-      result = sasl_client_step(conn, in, jlen, 
-				&client_interact, &out, &outlen);
+      result=sasl_client_step(conn, (const char *) in, jlen,
+			      &client_interact,
+			      &out, &outlen);
 
       if (result==SASL_INTERACT)
 	  result= fillin_interactions(env,obj,client_interact);
@@ -566,18 +610,26 @@ JNIEXPORT jbyteArray JNICALL Java_CyrusSasl_GenericClient_jni_1sasl_1client_1ste
 
   if (result == SASL_OK) {
       VL (("Setting complete\n"));
-      setcomplete(env, obj);
+      setcomplete(env,obj);
   }
 
   if (jarr) {
-      /* VL(("about to releasebytearrayelements"));
-       (*env)->ReleaseByteArrayElements(env, jarr, in, 0); */
+      VL(("about to releasebytearrayelements"));
+      (*env)->ReleaseByteArrayElements(env, jarr,in ,0);
+  }
+      
+  /* Because SASLv2 does not allow for persistance, we'll copy
+   * it here */
+  tmp = malloc(outlen);
+  if(!tmp) {
+      throwexception(env, SASL_NOMEM);
+      return NULL;
   }
 
-  VL(("about to newbytearray\n"));
-  /* make byte array to return with stuff to send to server */
-  arr = (*env)->NewByteArray(env, outlen);
-  (*env)->SetByteArrayRegion(env, arr, 0, outlen, (char *) out);
+  memcpy(tmp, out, outlen);
+  
+  arr=(*env)->NewByteArray(env,outlen);
+  (*env)->SetByteArrayRegion(env,arr, 0, outlen, (char *)tmp);
 
   VL(("returning arr\n"));
   return arr;
@@ -643,8 +695,9 @@ JNIEXPORT jbyteArray JNICALL Java_CyrusSasl_GenericCommon_jni_1sasl_1encode
 {
   sasl_conn_t *conn=(sasl_conn_t *) ptr;
   char *in = (*env)->GetByteArrayElements(env, jarr, 0);
-  char *out;
+  const char *out;
   unsigned int outlen;
+  char *tmp;
   int result;
   jbyteArray arr;
 
@@ -652,8 +705,18 @@ JNIEXPORT jbyteArray JNICALL Java_CyrusSasl_GenericCommon_jni_1sasl_1encode
   if (result!=SASL_OK)
     throwexception(env,result);
 
+  /* Because SASLv2 does not allow for persistance, we'll copy
+   * it here */
+  tmp = malloc(outlen);
+  if(!tmp) {
+      throwexception(env, SASL_NOMEM);
+      return NULL;
+  }
+
+  memcpy(tmp, out, outlen);
+  
   arr=(*env)->NewByteArray(env,outlen);
-  (*env)->SetByteArrayRegion(env,arr, 0, outlen, (char *) out);
+  (*env)->SetByteArrayRegion(env,arr, 0, outlen, (char *)tmp);
 
   return arr;
 }
@@ -667,8 +730,9 @@ JNIEXPORT jbyteArray JNICALL Java_CyrusSasl_GenericCommon_jni_1sasl_1decode
 
   sasl_conn_t *conn=(sasl_conn_t *) ptr;
   char *in = (*env)->GetByteArrayElements(env, jarr, 0);
-  char *out;
+  const char *out;
   unsigned int outlen=9;
+  char *tmp;
   int inlen=jlen;
   int result;
   jbyteArray arr;
@@ -677,8 +741,19 @@ JNIEXPORT jbyteArray JNICALL Java_CyrusSasl_GenericCommon_jni_1sasl_1decode
   if (result!=SASL_OK)
     throwexception(env,result);
 
+
+  /* Because SASLv2 does not allow for persistance, we'll copy
+   * it here */
+  tmp = malloc(outlen);
+  if(!tmp) {
+      throwexception(env, SASL_NOMEM);
+      return NULL;
+  }
+
+  memcpy(tmp, out, outlen);
+  
   arr=(*env)->NewByteArray(env,outlen);
-  (*env)->SetByteArrayRegion(env,arr, 0, outlen, (char *) out);
+  (*env)->SetByteArrayRegion(env,arr, 0, outlen, (char *)tmp);
 
   (*env)->ReleaseByteArrayElements(env, jarr, in,0);
 
@@ -712,7 +787,7 @@ JNIEXPORT jstring JNICALL Java_CyrusSasl_ServerFactory_jni_1sasl_1server_1getlis
   const char *pre = (*env)->GetStringUTFChars(env, jpre, 0);
   const char *sep = (*env)->GetStringUTFChars(env, jsep, 0);
   const char *suf = (*env)->GetStringUTFChars(env, jsuf, 0);
-  char *list;
+  const char *list;
   unsigned int plen;
   jstring ret;
 
@@ -737,19 +812,15 @@ JNIEXPORT void JNICALL Java_CyrusSasl_GenericCommon_jni_1sasl_1set_1server
    jint ptr, jbyteArray jarr, jint jport)
 {
   sasl_conn_t *conn=(sasl_conn_t *) ptr;
-  char *ip = (*env)->GetByteArrayElements(env, jarr, 0);
-  struct sockaddr_in addr;
+  unsigned char *ip = (*env)->GetByteArrayElements(env, jarr, 0);
+  char out[52];
   int result;
 
-  addr.sin_family = AF_INET;
-  memcpy(&addr.sin_addr, ip, 4);
-  addr.sin_port = htons(jport);
+  sprintf(out, "%d.%d.%d.%d;%d", ip[0], ip[1], ip[2], ip[3], (int)jport);
 
-  VL (("remote IP = %d %d %d %d\n",ip[0],ip[1],ip[2],ip[3]));
+  result=sasl_setprop(conn, SASL_IPREMOTEPORT, out);  
 
-  result=sasl_setprop(conn,   SASL_IP_REMOTE, &addr);  
-
-  VL(("Set IP_REMOTE: %d\n",result));
+  VL(("Set IP_REMOTE: %s: %d\n",out, result));
 
   /* if not set throw and exception */
   if (result!=SASL_OK)
@@ -764,19 +835,15 @@ JNIEXPORT void JNICALL Java_CyrusSasl_GenericCommon_jni_1sasl_1set_1client
    jint ptr, jbyteArray jarr, jint jport)
 {
   sasl_conn_t *conn=(sasl_conn_t *) ptr;
-  char *ip = (*env)->GetByteArrayElements(env, jarr, 0);
-  struct sockaddr_in addr;
+  unsigned char *ip = (*env)->GetByteArrayElements(env, jarr, 0);
+  char out[52];
   int result;
 
-  addr.sin_family = AF_INET;
-  memcpy(&addr.sin_addr, ip, 4);
-  addr.sin_port = htons(jport);
+  sprintf(out, "%d.%d.%d.%d;%d", ip[0], ip[1], ip[2], ip[3], (int)jport);
 
-  VL (("local IP = %d %d %d %d\n",ip[0],ip[1],ip[2],ip[3]));
+  result=sasl_setprop(conn, SASL_IPLOCALPORT, out);
 
-  result=sasl_setprop(conn,   SASL_IP_LOCAL, &addr);  
-
-  VL(("Set IP_LOCAL: %d\n",result));
+  VL(("Set IP_LOCAL: %s: %d\n",out, result));
 
   /* if not set throw and exception */
   if (result!=SASL_OK)
@@ -831,7 +898,7 @@ JNIEXPORT jint JNICALL Java_CyrusSasl_GenericCommon_jni_1sasl_1getSecurity
     sasl_conn_t *conn = (sasl_conn_t *) ptr;
     int *ssfp;
 
-    r = sasl_getprop(conn, SASL_SSF, (void **) &ssfp);
+    r = sasl_getprop(conn, SASL_SSF, (const void **) &ssfp);
     if (r != SASL_OK) {
 	throwexception(env, r);
     }
