@@ -1,6 +1,6 @@
 /* canonusr.c - user canonicalization support
  * Rob Siemborski
- * $Id: canonusr.c,v 1.6 2002/02/13 20:31:52 rjs3 Exp $
+ * $Id: canonusr.c,v 1.9 2002/07/18 01:21:27 ken3 Exp $
  */
 /* 
  * Copyright (c) 2001 Carnegie Mellon University.  All rights reserved.
@@ -118,6 +118,7 @@ int _sasl_canon_user(sasl_conn_t *conn,
 
 	/* Point the input copy at the stored buffer */
 	user = user_buf;
+	ulen = *lenp;
     }
 
     /* which plugin are we supposed to use? */
@@ -133,7 +134,10 @@ int _sasl_canon_user(sasl_conn_t *conn,
     }
     
     for(ptr = canonuser_head; ptr; ptr = ptr->next) {
-	if(!strcmp(plugin_name, ptr->name)) break;
+	/* A match is if we match the internal name of the plugin, or if
+	 * we match the filename (old-style) */
+	if((ptr->plug->name && !strcmp(plugin_name, ptr->plug->name))
+	   || !strcmp(plugin_name, ptr->name)) break;
     }
 
     /* We clearly don't have this one! */
@@ -173,21 +177,26 @@ int _sasl_canon_user(sasl_conn_t *conn,
     /* Set the appropriate oparams (lengths have already been set by lenp) */
     if(flags & SASL_CU_AUTHID) {
 	oparams->authid = conn->authid_buf;
-#ifndef macintosh
-	/* do auxprop lookups (server only) */
-	/* AUTHZID lookups do not occur until after authentication
-	   is complete */
-	if(sconn) {
-	    _sasl_auxprop_lookup(sconn->sparams, 0,
-			     oparams->authid, oparams->alen);
-	}
-#endif
     }
 
     if (flags & SASL_CU_AUTHZID) {
 	oparams->user = conn->user_buf;
     }
 
+#ifndef macintosh
+    /* do auxprop lookups (server only) */
+    if(sconn) {
+	if(flags & SASL_CU_AUTHID) {
+	    _sasl_auxprop_lookup(sconn->sparams, 0,
+				 oparams->authid, oparams->alen);
+	}
+	if(flags & SASL_CU_AUTHZID) {
+	    _sasl_auxprop_lookup(sconn->sparams, SASL_AUXPROP_AUTHZID,
+				 oparams->user, oparams->ulen);
+	}
+    }
+#endif
+
 
     RETURN(conn, SASL_OK);
 }
@@ -337,7 +346,7 @@ static sasl_canonuser_plug_t canonuser_internal_plugin = {
         0, /* features */
 	0, /* spare */
 	NULL, /* glob_context */
-	"INTERNAL", /* spare */
+	"INTERNAL", /* name */
 	NULL, /* canon_user_free */
 	_cu_internal_server,
 	_cu_internal_client,
