@@ -1,7 +1,7 @@
 /* saslutil.c
  * Rob Siemborski
  * Tim Martin
- * $Id: saslutil.c,v 1.34 2001/12/04 02:05:27 rjs3 Exp $
+ * $Id: saslutil.c,v 1.36 2002/07/24 17:22:26 rjs3 Exp $
  */
 /* 
  * Copyright (c) 2001 Carnegie Mellon University.  All rights reserved.
@@ -50,6 +50,10 @@
 #include <string.h>
 #include <assert.h>
 #include <ctype.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
+#include <errno.h>
 #ifdef HAVE_GETTIMEOFDAY
 #include <sys/time.h>
 #endif
@@ -75,8 +79,9 @@
 char *encode_table;
 char *decode_table;
 
+#define RPOOL_SIZE 3
 struct sasl_rand_s {
-    unsigned short pool[3];
+    unsigned short pool[RPOOL_SIZE];
     /* since the init time might be really bad let's make this lazy */
     int initialized; 
 };
@@ -278,26 +283,37 @@ int sasl_utf8verify(const char *str, unsigned len)
  * without specialized hardware etc...
  * thus, this is for nonce use only
  */
-void getranddata(unsigned short ret[3])
+void getranddata(unsigned short ret[RPOOL_SIZE])
 {
     long curtime;
     
-    memset(ret, 0, sizeof(unsigned short)*3);
+    memset(ret, 0, RPOOL_SIZE*sizeof(unsigned short));
 
 #ifdef DEV_RANDOM    
     {
-        FILE *f;
+	int fd;
 
-	if ((f = fopen(DEV_RANDOM, "r")) != NULL) {
-	    fread(ret, 1, sizeof(ret), f);
-	    fclose(f);
-	    return;
-        }
+	fd = open(DEV_RANDOM, O_RDONLY);
+	if(fd != -1) {
+	    unsigned char *buf = (unsigned char *)ret;
+	    size_t bytesread = 0;
+	    size_t bytesleft = RPOOL_SIZE*sizeof(unsigned short);
+	    
+	    do {
+		bytesread = read(fd, buf, bytesleft);
+		if(bytesread == -1 && errno == EINTR) continue;
+		else if(bytesread <= 0) break;
+		bytesleft -= bytesread;
+		buf += bytesread;
+	    } while(bytesleft != 0);
+		
+	    close(fd);
+	}
     }
 #endif
 
 #ifdef HAVE_GETPID
-    ret[0] = (unsigned short) getpid();
+    ret[0] ^= (unsigned short) getpid();
 #endif
 
 #ifdef HAVE_GETTIMEOFDAY
@@ -352,7 +368,7 @@ void sasl_randseed (sasl_rand_t *rpool, const char *seed, unsigned len)
     if (rpool == NULL) return;
 
     rpool->initialized = 1;
-    if (len > 6) len = 6;
+    if (len > 6) len = sizeof(unsigned short)*RPOOL_SIZE;
     for (lup = 0; lup < len; lup += 2)
 	rpool->pool[lup/2] = (seed[lup] << 8) + seed[lup + 1];
 }
@@ -412,7 +428,7 @@ void sasl_churn (sasl_rand_t *rpool, const char *data, unsigned len)
     randinit(rpool);
     
     for (lup=0; lup<len; lup++)
-	rpool->pool[lup % 3] ^= data[lup];
+	rpool->pool[lup % RPOOL_SIZE] ^= data[lup];
 }
 
 void sasl_erasebuffer(char *buf, unsigned len) {
