@@ -1,6 +1,6 @@
 /* Generic SASL plugin utility functions
  * Rob Siemborski
- * $Id: plugin_common.h,v 1.2 2001/12/04 02:06:49 rjs3 Exp $
+ * $Id: plugin_common.h,v 1.10 2002/04/30 17:45:33 ken3 Exp $
  */
 /* 
  * Copyright (c) 2001 Carnegie Mellon University.  All rights reserved.
@@ -96,20 +96,27 @@ int sasl_auxprop_plug_init(const sasl_utils_t *utils, \
 /* note: msg cannot include additional variables, so if you want to
  * do a printf-format string, then you need to call seterror yourself */
 #define SETERROR( utils, msg ) (utils)->seterror( (utils)->conn, 0, (msg) )
+
+#ifndef MEMERROR
 #define MEMERROR( utils ) \
     (utils)->seterror( (utils)->conn, 0, \
                        "Out of Memory in " __FILE__ " near line %d", __LINE__ )
+#endif
 
+#ifndef PARAMERROR
 #define PARAMERROR( utils ) \
     (utils)->seterror( (utils)->conn, 0, \
                        "Parameter Error in " __FILE__ " near line %d", __LINE__ )
+#endif
 
+#ifndef SASLINT_H
 typedef struct buffer_info 
 {
     char *data;
     unsigned curlen;   /* Current length of data in buffer */
     unsigned reallen;  /* total length of buffer (>= curlen) */
 } buffer_info_t;
+#endif
 
 int _plug_ipfromstring(const sasl_utils_t *utils, const char *addr,
 		       struct sockaddr *out, socklen_t outlen);
@@ -122,4 +129,43 @@ int _plug_strdup(const sasl_utils_t * utils, const char *in,
 void _plug_free_string(const sasl_utils_t *utils, char **str);
 void _plug_free_secret(const sasl_utils_t *utils, sasl_secret_t **secret);
 
+#define _plug_get_userid(utils, result, prompt_need) \
+	_plug_get_simple(utils, SASL_CB_USER, result, prompt_need)
+#define _plug_get_authid(utils, result, prompt_need) \
+	_plug_get_simple(utils, SASL_CB_AUTHNAME, result, prompt_need)
+int _plug_get_simple(const sasl_utils_t *utils, unsigned int id,
+		     const char **result, sasl_interact_t **prompt_need);
+
+int _plug_get_password(const sasl_utils_t *utils, sasl_secret_t **secret,
+		       unsigned int *iscopy, sasl_interact_t **prompt_need);
+
+int _plug_challenge_prompt(const sasl_utils_t *utils, unsigned int id,
+			   const char *challenge, const char *promptstr,
+			   const char **result, sasl_interact_t **prompt_need);
+
+int _plug_get_realm(const sasl_utils_t *utils, const char **availrealms,
+		    const char **realm, sasl_interact_t **prompt_need);
+
+int _plug_make_prompts(const sasl_utils_t *utils,
+		       sasl_interact_t **prompts_res,
+		       const char *user_prompt, const char *user_def,
+		       const char *auth_prompt, const char *auth_def,
+		       const char *pass_prompt, const char *pass_def,
+		       const char *echo_chal,
+		       const char *echo_prompt, const char *echo_def,
+		       const char *realm_chal,
+		       const char *realm_prompt, const char *realm_def);
+
+int _plug_decode(const sasl_utils_t *utils,
+		 void *context,
+		 const char *input, unsigned inputlen,
+		 char **output, unsigned *outputsize, unsigned *outputlen,
+		 int (*decode_pkt)(void *context,
+				   const char **input, unsigned *inputlen,
+				   char **output, unsigned *outputlen));
+
+int _plug_parseuser(const sasl_utils_t *utils,
+		    char **user, char **realm, const char *user_realm, 
+		    const char *serverFQDN, const char *input);
+
 #endif /* _PLUGIN_COMMON_H_ */
