@@ -18,6 +18,13 @@
 /* do we have SIA for plaintext password checking? */
 #undef HAVE_SIA
 
+/* do we have ldap support */
+#undef HAVE_LDAP
+
+/* do we want to enable the GSSAPI module */
+#undef AUTH_KRB5
+#undef KRB5_HEIMDAL
+
 /* Do we want to enable the experimental sasldb authentication module? */
 #undef AUTH_SASLDB
 
@@ -36,6 +43,12 @@
 /* do we have a sys/sio.h? */
 #undef HAVE_SYS_UIO_H
 
+/* do we have a crypt.h? */
+#undef HAVE_CRYPT_H
+
+/* do we have a sys/time.h */
+#undef HAVE_SYS_TIME_H
+
 /* Things SASLAUTHd doesn't really care about */
 #undef HAVE_SASLAUTHD
 #undef WITH_DES
@@ -43,7 +56,6 @@
 #undef STATIC_GSSAPIV2
 #undef STATIC_KERBEROS4
 #undef STATIC_PLAIN
-#undef HAVE_DB3_DB_H
 #undef SASL_BERKELEYDB
 #undef SASL_DB_PATH
 #undef SASL_GDBM
