@@ -28,7 +28,7 @@
  * END COPYRIGHT */
 
 #ifdef __GNUC__
-#ident "$Id: auth_krb5.c,v 1.3 2001/12/04 02:06:54 rjs3 Exp $"
+#ident "$Id: auth_krb5.c,v 1.7 2002/05/21 16:41:04 rjs3 Exp $"
 #endif
 
 /* ok, this is  wrong but the most convenient way of doing 
@@ -57,13 +57,142 @@
 #include <sys/stat.h>
 #include "auth_krb5.h"
 
-#ifdef DEADCODE
-extern int swap_bytes;			/* from libkrb.a   */
-#endif /* DEADCODE */
 /* END PUBLIC DEPENDENCIES */
 
-/* PRIVATE DEPENDENCIES */
+#define TF_DIR "/.tf"			/* Private tickets directory,   */
+					/* relative to mux directory    */
+
+char *tf_dir = NULL;
+
+int					/* R: -1 on failure, else 0 */
+auth_krb5_init (
+  /* PARAMETERS */
+  void					/* no parameters */
+  /* END PARAMETERS */
+  )
+{
 #ifdef AUTH_KRB5
+    int rc;
+    struct stat sb;
+
+    /* Allocate memory arena for the directory pathname. */
+    tf_dir = malloc(strlen(PATH_SASLAUTHD_RUNDIR) + strlen(TF_DIR)
+		    + 1024 + 2);
+    if (tf_dir == NULL) {
+	syslog(LOG_ERR, "auth_krb5_init malloc(tf_dir) failed");
+	return -1;
+    }
+    strcpy(tf_dir, PATH_SASLAUTHD_RUNDIR);
+    strcat(tf_dir, TF_DIR);
+
+    /* Check for an existing directory. */
+    rc = lstat(tf_dir, &sb);
+    if (rc == -1) {
+	if (errno == ENOENT) {
+	    /* Create the ticket file directory */
+	    rc = mkdir(tf_dir, 0700);
+	    if (rc == -1) {
+		syslog(LOG_ERR, "auth_krb5_init mkdir(%s): %m",
+		       tf_dir);
+		free(tf_dir);
+		tf_dir = NULL;
+		return -1;
+	    }
+	} else {
+	    rc = errno;
+	    syslog(LOG_ERR, "auth_krb5_init: %s: %m", tf_dir);
+	    free(tf_dir);
+	    tf_dir = NULL;
+	    return -1;
+	}
+    }
+
+    /* Make sure it's not a symlink. */
+    if (S_ISLNK(sb.st_mode)) {
+        syslog(LOG_ERR, "auth_krb5_init: %s: is a symbolic link", tf_dir);
+	free(tf_dir);
+	tf_dir = NULL;
+	return -1;
+    }
+    
+    return 0;
+
+#else
+    return -1;
+#endif
+}
+
+#ifdef AUTH_KRB5
+
+#ifdef KRB5_HEIMDAL
+
+char *					/* R: allocated response string */
+auth_krb5 (
+  /* PARAMETERS */
+  const char *user,			/* I: plaintext authenticator */
+  const char *password,			/* I: plaintext password */
+  const char *service __attribute__((unused)),
+  const char *realm __attribute__((unused))
+  /* END PARAMETERS */
+  )
+{
+    /* VARIABLES */
+    krb5_context context;
+    krb5_ccache ccache = NULL;
+    krb5_principal auth_user;
+    char * result;
+    char tfname[2048];
+    /* END VARIABLES */
+
+    if (!user|| !password) {
+	syslog(LOG_ERR, "auth_krb5: NULL password or username?");
+	return strdup("NO saslauthd internal NULL password or username");
+    }
+
+    if (krb5_init_context(&context)) {
+	syslog(LOG_ERR, "auth_krb5: krb5_init_context");
+	return strdup("NO saslauthd internal krb5_init_context error");
+    }
+    
+    if (krb5_parse_name (context, user, &auth_user)) {
+	krb5_free_context(context);
+	syslog(LOG_ERR, "auth_krb5: krb5_parse_name");
+	return strdup("NO saslauthd internal krb5_parse_name error");
+    }
+    
+    /* create a new CCACHE so we don't stomp on anything */
+    snprintf(tfname,sizeof(tfname), "%s/k5cc_%d_%d", tf_dir,
+	     getpid(), pthread_self());
+    if (krb5_cc_resolve(context, tfname, &ccache)) {
+	krb5_free_principal(context, auth_user);
+	krb5_free_context(context);
+	syslog(LOG_ERR, "auth_krb5: krb5_cc_resolve");
+	return strdup("NO saslauthd internal error");
+    }
+    
+    if (krb5_cc_initialize (context, ccache, auth_user)) {
+	krb5_free_principal(context, auth_user);
+	krb5_cc_destroy(context, ccache);
+	krb5_free_context(context);
+	syslog(LOG_ERR, "auth_krb5: krb5_cc_initialize");
+	return strdup("NO saslauthd internal error");
+    }
+
+    if (krb5_verify_user(context, auth_user, ccache, password, 1, NULL)) {
+	result = strdup("NO krb5_verify_user failed");
+    } else {
+	result = strdup("OK");
+    }
+
+    krb5_free_principal(context, auth_user);
+    krb5_cc_destroy(context, ccache);
+    krb5_free_context(context);
+
+    return result;
+}
+
+#else /* !KRB5_HEIMDAL */
+
 /* returns 0 for failure, 1 for success */
 static int k5support_verify_tgt(krb5_context context, 
 				krb5_ccache ccache) 
@@ -98,7 +227,9 @@ static int k5support_verify_tgt(krb5_context context,
     }
     thishost[BUFSIZ-1] = '\0';
     
+#ifdef KRB5_HEIMDAL
     krb5_data_zero(&packet);
+#endif
     k5_retcode = krb5_mk_req(context, &auth_context, 0, "host", 
 			     thishost, NULL, ccache, &packet);
     
@@ -120,17 +251,13 @@ static int k5support_verify_tgt(krb5_context context,
     /* all is good now */
     result = 1;
  fini:
+#ifndef KRB5_HEIMDAL
+    krb5_free_data_contents(context, &packet);
+#endif
     krb5_free_principal(context, server);
     
     return result;
 }
-#endif /* AUTH_KRB5 */
-/* END PRIVATE DEPENDENCIES */
-
-#define TF_DIR "/.tf"			/* Private tickets directory,   */
-					/* relative to mux directory    */
-
-
 
 /* FUNCTION: auth_krb5 */
 
@@ -138,8 +265,6 @@ static int k5support_verify_tgt(krb5_context context,
  * Authenticate against Kerberos IV.
  * END SYNOPSIS */
 
-#ifdef AUTH_KRB5
-
 char *					/* R: allocated response string */
 auth_krb5 (
   /* PARAMETERS */
@@ -235,6 +360,8 @@ auth_krb5 (
     return result;
 }
 
+#endif /* KRB5_HEIMDAL */
+
 #else /* ! AUTH_KRB5 */
 
 char *
