@@ -33,7 +33,7 @@ extern int g_argc;
 extern char **g_argv;
 extern char *path_db;
 extern authmech_t *proxymech;
-extern char *r_host;
+extern char *mech_option;
 extern char *r_service;
 extern int retry_read(int fd, void *buf, unsigned nbyte);
 extern int retry_writev (int fd, struct iovec *iov, int iovcnt);
