@@ -26,7 +26,7 @@
  * END COPYRIGHT */
 
 #ifdef __GNUC__
-#ident "$Id: mechanisms.h,v 1.4 2001/12/04 02:06:55 rjs3 Exp $"
+#ident "$Id: mechanisms.h,v 1.5 2002/05/03 18:59:30 rjs3 Exp $"
 #endif
 
 #include "saslauthd.h"
@@ -83,3 +83,9 @@ extern authmech_t *authmech;		/* auth mech daemon is using */
 #  define AUTH_PAM
 # endif
 #endif
+
+#ifndef AUTH_LDAP
+# ifdef HAVE_LDAP
+#  define AUTH_LDAP
+# endif
+#endif
